#ifndef WINSCREENSAVER_SYSTEM_H
#define WINSCREENSAVER_SYSTEM_H

#include <vector>
#include <string>

#include "math/vec.h"
#include "system/gl.h"
#include "system/system.h"

class WinScreenSaverSystem : public PSystem {
public:
	WinScreenSaverSystem() {reset(); }
	~WinScreenSaverSystem() {shutdown(); }

	virtual void init();
	virtual void shutdown();
	
	virtual void loop(void (*loopCallback)());
	void doLoop();
	virtual void requestExit();

	virtual void setInitDoubleBuffered(bool db);
	virtual void setInitPos(const vec2i &initPos);
	virtual void setInitSize(const vec2i &initSize);

	virtual bool getInScreenSaver() { return inScreenSaver; }
	virtual void setInScreenSaver(bool inScreenSaver);

	virtual const std::string &getArg(int index) const;
	virtual size_t getArgCount() const;

	HWND getHWnd() { return hwnd; }

	HWND getParentWnd() { return hParentWnd; }
	void setParentWnd(HWND hParentWnd_) { hParentWnd = hParentWnd_; }

public:
	bool initDBuf;
	int initBPP;
	double initTime;
	//this stuff might not be true for screensaver mode,
	//but hopefully I'll get it to the point that they apply to the config
	bool initFullScreen;
	bool inScreenSaver;
	const char *initTitle;
	vec2i initPos;
	vec2i initSize;

	std::vector<std::string> args;

	//for all those who want to swap/update display buffers manually inter-render
	void updateDisplayBuffers();
	
	void reset();

	//internal loop methods
	//for now I'm going to use a callback method... maybe switch it to an (abstract?) object later...
	void (*loopCallback)();

	void preUpdate();
	void postUpdate();
	
	//stored state variables
	bool dbuf;
	bool exitRequested;

	//system-specific
	HWND hwnd;				//current window
	HWND hParentWnd;		//parent window
	HDC hDC;				//device context
	HGLRC hRC;				//gl context
	DEVMODE devmodeBackup;	//stored mode when we started
};

#endif	//WINSCREENSAVER_SYSTEM_H
